#!/usr/bin/env python3
# -*- coding: UTF-8 -*-
"""
# File       : url.py
# Time       ：2020/8/19 15:15
# Author     ：Rodney Cheung
"""
import random
import socket
import urllib.request
from urllib.parse import urlparse

USER_AGENT = {
    "Android": [
        "Mozilla/5.0 (Linux; Android 6.0.1; RedMi Note 5 Build/RB3N5C; wv) AppleWebKit/537.36 (KHTML, like Gecko) "
        "Version/4.0 Chrome/68.0.3440.91 Mobile Safari/537.36",
        "Mozilla/5.0 (Linux; Android 7.1.2; AFTMM Build/NS6265; wv) AppleWebKit/537.36 (KHTML, like Gecko) "
        "Version/4.0 Chrome/70.0.3538.110 Mobile Safari/537.36",
        "Mozilla/5.0 (Linux; Android 6.0; MotoG3 Build/MPIS24.65-33.1-2-16) AppleWebKit/537.36 (KHTML, like Gecko) "
        "Chrome/67.0.3396.87 Mobile Safari/537.36 "
    ],
    "iOS": [
        "Mozilla/5.0 (iPhone; CPU iPhone OS 12_1 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) "
        "CriOS/70.0.3538.75 Mobile/15E148 Safari/605.1",
        "Mozilla/5.0 (iPhone; CPU iPhone OS 13_1 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) "
        "CriOS/78.0.3904.84 Mobile/15E148 Safari/604.1",
        "Mozilla/5.0 (iPhone; CPU iPhone OS 12_0 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) "
        "CriOS/69.0.3497.105 Mobile/15E148 Safari/605.1",
    ],
}


def get_ip_from_url(url):
    netloc = urlparse(url).netloc
    domain = netloc.split(":")[0]
    addr = socket.getaddrinfo(domain, 'http')
    return addr[0][4][0]


def get_domain_from_url(url: str) -> str:
    netloc = urlparse(url).netloc
    domain = netloc.split(":")[0]
    return domain


def check_url_access(url: str, user_agent: tuple) -> bool:
    """
    check if url accessible
    Args:
        url: target url
        user_agent: user agent,use USER_AGENT['Android'] or USER_AGENT['iOS']

    Returns:
        url accessible
    """
    opener = urllib.request.build_opener()
    opener.addheaders = [('User-agent',
                          user_agent[random.randint(
                              0,
                              len(user_agent) - 1)])]
    try:
        opener.open(url)
        return True
    except Exception as e:
        print("Current URL[{0}] connect failed!".format(url), e)
        return False



# !/usr/bin/env python3
# -*-coding:utf-8 -*-

"""
# File       : apk.py
# Time       ：2020/9/18 15:21
# Author     ：Rodney Cheung
"""
import re

from wisbec.android.aapt import Aapt


class Apk:
    @classmethod
    def init(cls, aapt_path: str):
        Aapt.init(aapt_path)

    @classmethod
    def get_pkg_name(cls, apk_path: str) -> str:
        code, out, err = Aapt.dump_label(apk_path)
        if code != 0:
            return ''
        package_name_pattern: re.Pattern = re.compile(r'package: name=\'\S+\'')
        matched: re.Match = package_name_pattern.search(out)
        if matched is None:
            return ''
        return out[matched.span()[0] + 15: matched.span()[1] - 1]



# !/usr/bin/env python3
# -*-coding:utf-8 -*-

"""
# File       : file_writer.py
# Time       ：2020/8/19 16:05
# Author     ：Rodney Cheung
"""


def write_file(path, mode, data):
    with open(path, mode) as f:
        f.write(data)



#!/usr/bin/env python3
# -*- coding: UTF-8 -*-
"""
@Time    : 8/18/20 1:56 PM
@Author  : Rodney Cheung
@File    : db.py
"""
import sqlite3

import pymysql
from sshtunnel import SSHTunnelForwarder


class DataBase(object):
    def __init__(self):
        self.connection = None

    def connect(self):
        pass

    def close(self):
        self.connection.commit()
        self.connection.close()

    def __del__(self):
        self.close()


class Sqlite3DataBase(DataBase):
    def __init__(self, db_path):
        super(Sqlite3DataBase, self).__init__()
        self.db_path = db_path

    def connect(self):
        self.connection = sqlite3.connect(self.db_path,
                                          check_same_thread=False)


class MySqlDataBase(DataBase):
    def __init__(self,
                 username,
                 password,
                 host,
                 port,
                 database_name,
                 raise_on_warnings=True):
        super(MySqlDataBase, self).__init__()
        self.username = username
        self.password = password
        self.host = host
        self.port = port
        self.database_name = database_name
        self.raise_on_warnings = raise_on_warnings
        self.tunnel = None

    def connect(self):
        self.connection = pymysql.connect(host=self.host,
                                          port=self.port,
                                          user=self.username,
                                          password=self.password,
                                          db=self.database_name)

    def connect_by_ssh_tunnel(self, ssh_host, ssh_port, ssh_username,
                              ssh_private_key):
        self.tunnel = SSHTunnelForwarder(
            (ssh_host, ssh_port),
            ssh_username=ssh_username,
            ssh_pkey=ssh_private_key,
            remote_bind_address=(self.host, self.port))
        self.tunnel.start()
        self.connection = pymysql.connect(host='127.0.0.1',
                                          port=self.tunnel.local_bind_port,
                                          user=self.username,
                                          password=self.password,
                                          db=self.database_name)

    def close(self):
        self.connection.commit()
        self.connection.close()
        if self.tunnel is not None:
            self.tunnel.stop()



import os

from wisbec.console import shell


def calc_hash_name_by_openssl_x509(cacert_path):
    args = ['openssl', 'x509', '-subject_hash_old', '-in', cacert_path]
    code, out, err = shell.exec_cmd(args)
    if code != 0:
        return None
    return '{}.0'.format(out.split(os.linesep)[0])



# !/usr/bin/env python3
# -*-coding:utf-8 -*-

"""
# File       : shell.py
# Time       ：2020/8/19 15:15
# Author     ：Rodney Cheung
"""
import os
import subprocess

from wisbec import path


def __decode(content):
    try:
        return content.decode('utf-8')
    except UnicodeDecodeError:
        return content.decode('gb2312')


def exec_cmd(cmd, cwd=None, stdout=subprocess.PIPE, stderr=subprocess.PIPE, shell=False):
    try:
        subp = subprocess.Popen(cmd, shell=shell, stdout=stdout, stderr=stderr, cwd=cwd)
    except Exception as e:
        print('exec cmd Exception: ' + str(e))
        return None, None, None
    out, err = subp.communicate()
    normal = None
    error = None
    if out is not None:
        normal = __decode(out)
    if err is not None:
        error = __decode(err)
    return subp.returncode, normal, error


def get_current_shell():
    current_shell = os.environ['SHELL']
    return current_shell.split('/')[-1]


def write_env(target_path):
    sh = get_current_shell()
    if sh == 'zsh':
        sh_rc_file = os.path.join(path.home_dir(), '.zshrc')
    else:
        sh_rc_file = os.path.join(path.home_dir(), '.bashrc')
    cmd = 'export PATH=$PATH:{}'.format(target_path)
    with open(sh_rc_file, 'a+') as f:
        f.write('\n')
        f.write(cmd)



import os

from wisbec import system


def home_dir() -> str:
    current_os = system.current_os
    if current_os == 'darwin' or current_os == 'linux':
        return os.environ['HOME']
    elif current_os == 'win32':
        return os.environ['USERPROFILE']
    else:
        return ''



import sys

current_os = sys.platform

WINDOWS = 'win32'
MAC_OS = 'darwin'
LINUX = 'linux'


def get_newline_ch() -> str:
    if current_os == WINDOWS:
        return '\r\n'
    else:
        return '\n'


def is_windows() -> bool:
    return current_os == WINDOWS


def is_linux() -> bool:
    return current_os == LINUX


def is_mac_os() -> bool:
    return current_os == MAC_OS



# !/usr/bin/env python3
# -*-coding:utf-8 -*-

"""
# File       : aapt.py
# Time       ：2020/9/18 15:29
# Author     ：Rodney Cheung
"""
from wisbec.console.shell import exec_cmd


class Aapt:
    c_aapt_path: str = ''

    @classmethod
    def init(cls, aapt_path: str):
        cls.c_aapt_path = aapt_path

    @classmethod
    def exec(cls, *args):
        if cls.c_aapt_path == '':
            cmd = ['aapt'] + list(args)
        else:
            cmd = [cls.c_aapt_path] + list(args)
        return exec_cmd(cmd)

    @classmethod
    def dump(cls, *args):
        return cls.exec('dump', *args)

    @classmethod
    def dump_label(cls, apk_path: str):
        return cls.dump('badging', apk_path)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
@Time    : 8/18/20 1:56 PM
@Author  : Rodney Cheung
@File    : log.py
"""

import io
import logging
import os
import time
import traceback

from colorlog import ColoredFormatter

from wisbec.file import filesystem


class MyLogger(logging.Logger):
    def findCaller(self, stack_info=False, stack_level=1):
        n_frames_upper = 2
        f = logging.currentframe()
        for _ in range(n_frames_upper):  # <-- correct frame
            if f is not None:
                f = f.f_back
        rv = "(unknown file)", 0, "(unknown function)", None
        while hasattr(f, "f_code"):
            co = f.f_code
            filename = os.path.normcase(co.co_filename)
            if filename == logging._srcfile:
                f = f.f_back
                continue
            sinfo = None
            if stack_info:
                sio = io.StringIO()
                sio.write('Stack (most recent call last):\n')
                traceback.print_stack(f, file=sio)
                sinfo = sio.getvalue()
                if sinfo[-1] == '\n':
                    sinfo = sinfo[:-1]
                sio.close()
            rv = (co.co_filename, f.f_lineno, co.co_name, sinfo)
            break
        return rv


class Log:
    logging.setLoggerClass(MyLogger)
    console_logger = None
    console_handler = None
    file_log_formatter = logging.Formatter(
        "[%(levelname)s] %(asctime)s %(funcName)s %(lineno)d: %(message)s")
    console_log_formatter = ColoredFormatter(
        "%(log_color)s[%(levelname)s] %(asctime)s %(funcName)s %(lineno)d : %(message)s",
        datefmt=None,
        reset=True,
        log_colors={
            'DEBUG': 'cyan',
            'INFO': 'green',
            'WARNING': 'yellow',
            'ERROR': 'red',
            'CRITICAL': 'red,bg_white',
        },
        secondary_log_colors={},
        style='%')
    is_log_to_file = False
    is_log_to_console = True
    file_loggers = {}

    log_dir = None
    log_debug_path = None
    log_info_path = None
    log_err_path = None
    log_warning_path = None
    log_critical_path = None
    log_path_dict = {}

    file_handler_created_flag = {
        logging.DEBUG: False,
        logging.WARNING: False,
        logging.ERROR: False,
        logging.CRITICAL: False,
        logging.INFO: False
    }

    @classmethod
    def __init_log_dir(cls, log_dir):
        filesystem.create_directories(log_dir)
        cls.log_dir = log_dir
        cls.log_debug_path = os.path.join(log_dir, "debug.log")
        cls.log_info_path = os.path.join(log_dir, "info.log")
        cls.log_err_path = os.path.join(log_dir, "error.log")
        cls.log_warning_path = os.path.join(log_dir, "warning.log")
        cls.log_critical_path = os.path.join(log_dir, "critical.log")
        cls.log_path_dict = {
            logging.DEBUG: cls.log_debug_path,
            logging.WARNING: cls.log_warning_path,
            logging.ERROR: cls.log_err_path,
            logging.CRITICAL: cls.log_critical_path,
            logging.INFO: cls.log_info_path
        }

    @classmethod
    def __init_log_handler(
            cls,
            is_log_to_file,
            is_log_to_console,
            console_log_name,
            console_log_level):
        cls.is_log_to_console = is_log_to_console
        if is_log_to_console:
            cls.console_handler = logging.StreamHandler()
            cls.console_handler.setFormatter(cls.console_log_formatter)
            cls.console_handler.setLevel(console_log_level)
            cls.console_logger = logging.getLogger(console_log_name)
            cls.console_logger.setLevel(logging.DEBUG)
            cls.console_logger.addHandler(cls.console_handler)
        cls.is_log_to_file = is_log_to_file

    @classmethod
    def init_logger(cls,
                    log_dir=os.path.join(os.getcwd(),
                                         "runtime", "log",
                                         time.strftime("%Y-%m-%d %H:%M:%S",
                                                       time.localtime())),
                    is_log_to_file=True,
                    is_log_to_console=True,
                    console_log_name='console_log',
                    console_log_level=logging.DEBUG,
                    ):
        """
        init logger
        Args:
            log_dir: log save path
            is_log_to_file:is output log to file
            is_log_to_console:is output log to console
            console_log_name: logger name
            console_log_level:minimum log level,if not set,all level logs will be recorded
        Returns:
            None
        """
        cls.__init_log_dir(log_dir)
        cls.__init_log_handler(
            is_log_to_file,
            is_log_to_console,
            console_log_name,
            console_log_level)

    @classmethod
    def __position_format(cls, msg: str, *args, **kwargs) -> str:
        # position_str = '{}'
        # position_str_list = list()
        # arg_len = len(args)
        # while arg_len > 0:
        #     position_str_list.append(position_str)
        #     arg_len -= 1
        # return "".join(position_str_list).format(*args)
        return msg.format(*args, **kwargs)

    @classmethod
    def debug(cls, msg: str, *args, **kwargs):
        data = cls.__position_format(msg, *args, **kwargs)
        if cls.is_log_to_console:
            cls.console_logger.debug(data)
        if cls.is_log_to_file:
            if not cls.file_handler_created_flag[logging.DEBUG]:
                cls.add_file_handler(cls.log_debug_path, logging.DEBUG)
                cls.file_handler_created_flag[logging.DEBUG] = True
            cls.file_loggers[logging.DEBUG].debug(data)

    @classmethod
    def info(cls, msg: str, *args, **kwargs):
        data = cls.__position_format(msg, *args, **kwargs)
        if cls.is_log_to_console:
            cls.console_logger.info(data)
        if cls.is_log_to_file:
            if not cls.file_handler_created_flag[logging.INFO]:
                cls.add_file_handler(cls.log_info_path, logging.INFO)
                cls.file_handler_created_flag[logging.INFO] = True
            cls.file_loggers[logging.INFO].info(data)

    @classmethod
    def warning(cls, msg: str, *args, **kwargs):
        data = cls.__position_format(msg, *args, **kwargs)
        if cls.is_log_to_console:
            cls.console_logger.warning(data)
        if cls.is_log_to_file:
            if not cls.file_handler_created_flag[logging.WARNING]:
                cls.add_file_handler(cls.log_warning_path, logging.WARNING)
                cls.file_handler_created_flag[logging.WARNING] = True
            cls.file_loggers[logging.WARNING].warning(data)

    @classmethod
    def error(cls, msg: str, *args, **kwargs):
        data = cls.__position_format(msg, *args, **kwargs)
        if cls.is_log_to_console:
            cls.console_logger.error(data)
        if cls.is_log_to_file:
            if not cls.file_handler_created_flag[logging.ERROR]:
                cls.add_file_handler(cls.log_err_path, logging.ERROR)
                cls.file_handler_created_flag[logging.ERROR] = True
            cls.file_loggers[logging.ERROR].error(data)

    @classmethod
    def critical(cls, msg: str, *args, **kwargs):
        data = cls.__position_format(msg, *args, **kwargs)
        if cls.is_log_to_console:
            cls.console_logger.critical(data)
        if cls.is_log_to_file:
            if not cls.file_handler_created_flag[logging.CRITICAL]:
                cls.add_file_handler(cls.log_critical_path, logging.CRITICAL)
                cls.file_handler_created_flag[logging.CRITICAL] = True
            cls.file_loggers[logging.CRITICAL].critical(data)

    @classmethod
    def add_file_handler(cls, log_file, log_level):
        file_handler = logging.FileHandler(log_file)
        file_handler.setFormatter(cls.file_log_formatter)
        file_handler.setLevel(log_level)
        file_logger = logging.getLogger(str(log_level))
        file_logger.setLevel(logging.DEBUG)
        file_logger.addHandler(file_handler)
        cls.file_loggers.update({log_level: file_logger})

    @classmethod
    def set_console_log_level(cls, log_level):
        cls.console_handler.setLevel(log_level)

    @classmethod
    def close(cls):
        cls.clear_file_handler()

    @classmethod
    def clear_file_handler(cls):
        for file_logger in cls.file_loggers:
            cls.file_loggers[file_logger].handlers[0].close()
            cls.file_handler_created_flag[file_logger] = False
        cls.file_loggers.clear()



# !/usr/bin/env python3
# -*-coding:utf-8 -*-

"""
# File       : interface.py
# Time       ：2020/8/24 08:48
# Author     ：Rodney Cheung
"""
import re
import sys

from wisbec.console import shell


class Interface:
    @classmethod
    def __get_interface_from_ifconfig(cls) -> list:
        """
        enx000ec6c0c623: flags=4099<UP,BROADCAST,MULTICAST>  mtu 1500
                ether 00:0e:c0:c0:56:23  txqueuelen 1000  (Ethernet)
                RX packets 0  bytes 0 (0.0 B)
                RX errors 0  dropped 0  overruns 0  frame 0
                TX packets 0  bytes 0 (0.0 B)
                TX errors 0  dropped 0 overruns 0  carrier 0  collisions 0
        lo: flags=73<UP,LOOPBACK,RUNNING>  mtu 65536
                inet 127.0.0.1  netmask 255.0.0.0
                inet6 ::1  prefixlen 128  scopeid 0x10<host>
                loop  txqueuelen 1000  (Local Loopback)
                RX packets 1134271  bytes 1046040386 (1.0 GB)
                RX errors 0  dropped 0  overruns 0  frame 0
                TX packets 1134271  bytes 1046040386 (1.0 GB)
                TX errors 0  dropped 0 overruns 0  carrier 0  collisions 0
        """
        list_interface = []
        code, out, err = shell.exec_cmd('ifconfig')
        out = out.replace('\n\n', '\n')
        list_blocks = re.split('\n[a-zA-Z0-9]', out)
        for block in list_blocks:
            if ('inet ' not in block) or ('loop ' in block):
                continue
            iface_name = re.search(r'(\S+):', block).groups()[0]
            iface_ip = re.search(r'inet (\S+) ', block).groups()[0].replace('addr:', '')
            list_interface.append((iface_name, iface_ip))
        return list_interface

    @classmethod
    def __get_interface_from_ipconfig(cls) -> list:
        """
        Windows IP Configuration

        Ethernet adapter 以太网:

           Media State . . . . . . . . . . . : Media disconnected
           Connection-specific DNS Suffix  . :

        Wireless LAN adapter 本地连接* 2:

           Media State . . . . . . . . . . . : Media disconnected
           Connection-specific DNS Suffix  . :

        Wireless LAN adapter WLAN:

           Connection-specific DNS Suffix  . :
           IPv4 Address. . . . . . . . . . . : 10.251.0.49
           Subnet Mask . . . . . . . . . . . : 255.255.254.0
           Default Gateway . . . . . . . . . : 10.251.0.1

        Ethernet adapter 蓝牙网络连接:

           Media State . . . . . . . . . . . : Media disconnected
           Connection-specific DNS Suffix  . :
        """
        interface_list = []
        code, out, err = shell.exec_cmd(['cmd.exe', '/c', 'chcp 65001 & ipconfig'])
        if code != 0:
            return interface_list
        out = re.sub(r'(\r\n){2,}', r'\r\n', out)
        list_blocks = re.split(r'\r\n[a-zA-Z0-9]', out)
        for block in list_blocks:
            if 'IPv4 Address' not in block:
                continue
            iface_name_group_span = re.search(r'(\S+):', block).span()
            iface_name = block[iface_name_group_span[0]:iface_name_group_span[1]].replace(':', '')
            search_res = re.search(r'IPv4 Address(. )+: \S+', block)
            if search_res is None:
                continue
            search_res_span = search_res.span()
            ip_addr = block[search_res_span[0]:search_res_span[1]]
            ip_addr = ip_addr.split(': ')[-1]
            interface_list.append((iface_name, ip_addr))
        return interface_list

    @classmethod
    def get_active_interfaces(cls) -> list:
        """
        get active net interface,support mac os,linux an windows
        Returns:
            [(iface_name1,iface_ip1),...]
        """
        current_os = sys.platform
        if current_os == 'win32':
            return cls.__get_interface_from_ipconfig()
        elif current_os == 'darwin' or current_os == 'linux':
            return cls.__get_interface_from_ifconfig()
        else:
            return []



#!/usr/bin/env python3
# -*- coding: UTF-8 -*-

'''
@Author: Rodney Cheung
@Date: 2020-05-13 09:51:55
@LastEditors: Rodney Cheung
@LastEditTime: 2020-06-29 10:50:52
@FilePath: /python-wisbec/core/__init__.py
'''
